/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;

/**
 * Address details conform EDI-Teelt.
 * 
 * @author johan & Merijn Zengers
 */
public class Address implements Serializable{
    public static final String PROP_STREET_NAME = "streetName";
    private String streetName; // an..30
    public static final String PROP_HOUSE_NUMBER = "houseNumber";
    private Integer houseNumber; // n..6
    public static final String PROP_POSTAL_CODE = "postalCode";
    private String postalCode; // an..6
    public static final String PROP_CITY = "city";
    private String city; // an..30
    public static final String PROP_COUNTRY = "country";
    private Country country; // an..3: 3 character ISO code
    public static final String PROP_ADDITION = "addition";
    private String addition;
    
    private transient PropertyChangeSupport propertyChangeSupport;

    public Address() {
    }

    public Address(String streetName, int houseNumber, String postalCode, String city, Country country) {
        this.streetName = streetName;
        this.houseNumber = houseNumber;
        this.postalCode = postalCode;
        this.city = city;
        this.country = country;
    }
    
    private PropertyChangeSupport getPropertyChangeSupport(){
        if(propertyChangeSupport == null){
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }
    
    /**
     * Add a {@link PropertyChangeListener} to this address
     * @param listener 
     */
    public void addPropertyChangeListener(PropertyChangeListener listener){
        getPropertyChangeSupport().addPropertyChangeListener(listener);
    }
    
    /**
     * Add a {@link PropertyChangeListener} to this address for a specific property
     * @param prop
     * @param listener 
     */
    public void addPropertyChangeListener(String prop, PropertyChangeListener listener){
        getPropertyChangeSupport().addPropertyChangeListener(prop, listener);
    }
    
    /**
     * remove a {@link PropertyChangeListener} from this address
     * @param listener 
     */
    public void removePropertyChangeListener(PropertyChangeListener listener){
        getPropertyChangeSupport().removePropertyChangeListener(listener);
    }
    
    /**
     *  remove a {@link PropertyChangeListener} from this address for a specific property
     * @param property
     * @param listener 
     */
    public void removePropertyChangeListener(String property, PropertyChangeListener listener){
        getPropertyChangeSupport().removePropertyChangeListener(property, listener);
    }

    /**
     * Get the City of this address
     * @return 
     */
    public String getCity() {
        return city;
    }

    /**
     * Set the city of this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param city 
     */
    public void setCity(String city) {
        String old = this.city;
        this.city = city;
        getPropertyChangeSupport().firePropertyChange(PROP_CITY, old, city);
    }

    /**
     * Get the {@link Country} of this address
     * @return 
     */
    public Country getCountry() {
        return country;
    }

    /**
     * Set the {@link Country} of this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param country 
     */
    public void setCountry(Country country) {
        Country old = this.country;
        this.country = country;
        getPropertyChangeSupport().firePropertyChange(PROP_COUNTRY, old, country);
    }

    /**
     * Get the house number of this address
     * @return 
     */
    public Integer getHouseNumber() {
        return houseNumber;
    }

    /**
     * Set the house number for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param houseNumber 
     */
    public void setHouseNumber(Integer houseNumber) {
        Integer old = this.houseNumber;
        this.houseNumber = houseNumber;
        getPropertyChangeSupport().firePropertyChange(PROP_HOUSE_NUMBER, old, houseNumber);
    }

    /**
     * Get the postal code of this address 
     * @return 
     */
    public String getPostalCode() {
        return postalCode;
    }

    /**
     * Set the postal code for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param postalCode 
     */
    public void setPostalCode(String postalCode) {
        String old = this.postalCode;
        this.postalCode = postalCode;
        getPropertyChangeSupport().firePropertyChange(PROP_POSTAL_CODE, old, postalCode);
    }

    /**
     * Get the street name of this address
     * @return 
     */
    public String getStreetName() {
        return streetName;
    }

    /**
     * Set the street name for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param streetName 
     */
    public void setStreetName(String streetName) {
        String old = this.streetName;
        this.streetName = streetName;
        getPropertyChangeSupport().firePropertyChange(PROP_STREET_NAME, old, streetName);
    }

    /**
     * Get the addition for this address
     *
     * @return 
     */    
    public String getAddition() {
        return addition;
    }

    /**
     * Set the addition for this address
     * Fires a {@link PropertyChangeEvent} on change
     * @param addition 
     */
    public void setAddition(String addition) {
        String old = this.addition;
        this.addition = addition;
        getPropertyChangeSupport().firePropertyChange(PROP_ADDITION, old, addition);
    }
    
    

}
